@@ -10,7 +10,7 @@ class Matrix:
 
     @property
     def size(self):
-        return (len(self.strings[0]), len(self.strings)) if self.strings else (0, 0)
+        return (len(self.strings), len(self.strings[0])) if self.strings else (0, 0)
 
     def clone(self):
         return Matrix(*self.strings)
@@ -19,6 +19,16 @@ class Matrix:
     def filled(e, m, n):
         return Matrix(*((e, ) * m, ) * n)
 
+    @staticmethod
+    def generated(func, m, n):
+        result = Matrix.zero(m, n)
+
+        for y in range(m):
+            for x in range(n):
+                result[y, x] = func(y, x)
+
+        return result
+
     @staticmethod
     def zero(m, n):
         return Matrix.filled(0, m, n)
@@ -28,7 +38,7 @@ class Matrix:
             assert len(key) == 2
             return self.strings[key[0]][key[1]]
         if isinstance(key, Matrix):
-            assert key.size == (2, 1)
+            assert key.size == (1, 2)
             return self[key.strings[0][0], key.strings[0][1]]
         raise TypeError
 
@@ -38,7 +48,7 @@ class Matrix:
             self.strings[key[0]][key[1]] = value
             return
         if isinstance(key, Matrix):
-            assert key.size == (2, 1)
+            assert key.size == (1, 2)
             self[key.strings[0][0], key.strings[0][1]] = value
             return
         raise TypeError
@@ -72,8 +82,18 @@ class Matrix:
         return self
 
     def __imul__(self, other):
-        for y, x, _ in self.enumerate():
-            self[y, x] *= other
+        if isinstance(other, Matrix):
+            assert self.size[1] == other.size[0]
+
+            old_strings = self.strings
+            self.strings = [[0] * other.size[1] for _ in range(self.size[0])]
+
+            for y, x, _ in self.enumerate():
+                for i in range(other.size[0]):
+                    self[y, x] += old_strings[y][i] * other[i, x]
+        else:
+            for y, x, _ in self.enumerate():
+                self[y, x] *= other
         return self
 
     def __itruediv__(self, other):
@@ -104,9 +124,7 @@ class Matrix:
         return result
 
     def __rmul__(self, other):
-        if not isinstance(other, Matrix):
-            return self * other
-        raise TypeError
+        return self * other
 
     def __neg__(self):
         result = self.clone()
@@ -121,30 +139,6 @@ class Matrix:
     def __repr__(self):
         return str(self)
 
-#     def __neg__(self):
-#         return Vector(
-#             -self.x,
-#             -self.y
-#         )
-#
-#     def __sub__(self, other):
-#         return self + -other
-#
-#     def __mul__(self, other):
-#         if isinstance(other, Vector):
-#             return self.x * other.x + self.y * other.y
-#
-#         return Vector(
-#             self.x * other,
-#             self.y * other
-#         )
-#
-#     def __rmul__(self, other):
-#         return self * other
-#
-#     def __truediv__(self, other):
-#         return self * (1 / other)
-#
 #     def squared_magnitude(self):
 #         return self.x ** 2 + self.y ** 2
 #
